@@ -10,6 +10,12 @@
  * 2 of the License, or (at your option) any later version.
  */
 
+/*
+ * TODO:
+ * handle 2 drives
+ * handle GCR disks
+ */
+
 #include <linux/stddef.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -22,12 +28,15 @@
 #include <asm/prom.h>
 #include <asm/uaccess.h>
 #include <asm/mediabay.h>
+#include <asm/feature.h>
 
 #define MAJOR_NR	FLOPPY_MAJOR
 #include <linux/blk.h>
 
-static int floppy_blocksizes[2] = {512};
-static int floppy_sizes[2] = {2880};
+static int floppy_blocksizes[2] = {512,512};
+static int floppy_sizes[2] = {2880,2880};
+
+#define MAX_FLOPPIES	2
 
 enum swim_state {
 	idle,
@@ -49,18 +58,18 @@ enum swim_state {
  */
 struct swim3 {
 	REG(data);
-	REG(usecs);		/* counts down at 1MHz */
+	REG(timer);		/* counts down at 1MHz */
 	REG(error);
 	REG(mode);
 	REG(select);		/* controls CA0, CA1, CA2 and LSTRB signals */
-	REG(reg5);
+	REG(setup);
 	REG(control);		/* writing bits clears them */
 	REG(status);		/* writing bits sets them in control */
 	REG(intr);
 	REG(nseek);		/* # tracks to seek */
 	REG(ctrack);		/* current track number */
 	REG(csect);		/* current sector number */
-	REG(ssize);		/* sector size code?? */
+	REG(gap3);		/* size of gap 3 in track format */
 	REG(sector);		/* sector # to read or write */
 	REG(nsect);		/* # sectors to read or write */
 	REG(intr_enable);
@@ -75,21 +84,46 @@ struct swim3 {
 
 /* Bits in control register */
 #define DO_SEEK		0x80
+#define FORMAT		0x40
 #define SELECT		0x20
 #define WRITE_SECTORS	0x10
-#define SCAN_TRACK	0x08
+#define DO_ACTION	0x08
+#define DRIVE2_ENABLE	0x04
 #define DRIVE_ENABLE	0x02
 #define INTR_ENABLE	0x01
 
 /* Bits in status register */
+#define FIFO_1BYTE	0x80
+#define FIFO_2BYTE	0x40
+#define ERROR		0x20
 #define DATA		0x08
+#define RDDATA		0x04
+#define INTR_PENDING	0x02
+#define MARK_BYTE	0x01
 
 /* Bits in intr and intr_enable registers */
-#define ERROR		0x20
+#define ERROR_INTR	0x20
 #define DATA_CHANGED	0x10
 #define TRANSFER_DONE	0x08
 #define SEEN_SECTOR	0x04
 #define SEEK_DONE	0x02
+#define TIMER_DONE	0x01
+
+/* Bits in error register */
+#define ERR_DATA_CRC	0x80
+#define ERR_ADDR_CRC	0x40
+#define ERR_OVERRUN	0x04
+#define ERR_UNDERRUN	0x01
+
+/* Bits in setup register */
+#define S_SW_RESET	0x80
+#define S_GCR_WRITE	0x40
+#define S_IBM_DRIVE	0x20
+#define S_TEST_MODE	0x10
+#define S_FCLK_DIV2	0x08
+#define S_GCR		0x04
+#define S_COPY_PROT	0x02
+#define S_INV_WDATA	0x01
 
 /* Select values for swim3_action */
 #define SEEK_POSITIVE	0
@@ -97,14 +131,18 @@ struct swim3 {
 #define STEP		1
 #define MOTOR_ON	2
 #define MOTOR_OFF	6
+#define INDEX		3
 #define EJECT		7
+#define SETMFM		9
+#define SETGCR		13
 
 /* Select values for swim3_select and swim3_readbit */
 #define STEP_DIR	0
 #define STEPPING	1
 #define MOTOR_ON	2
-#define RELAX		3
+#define RELAX		3	/* also eject in progress */
 #define READ_DATA_0	4
+#define TWOMEG_DRIVE	5
 #define SINGLE_SIDED	6
 #define DRIVE_PRESENT	7
 #define DISK_IN		8
@@ -112,7 +150,25 @@ struct swim3 {
 #define TRACK_ZERO	10
 #define TACHO		11
 #define READ_DATA_1	12
+#define MFM_MODE	13
 #define SEEK_COMPLETE	14
+#define ONEMEG_MEDIA	15
+
+/* Definitions of values used in writing and formatting */
+#define DATA_ESCAPE	0x99
+#define GCR_SYNC_EXC	0x3f
+#define GCR_SYNC_CONV	0x80
+#define GCR_FIRST_MARK	0xd5
+#define GCR_SECOND_MARK	0xaa
+#define GCR_ADDR_MARK	"\xd5\xaa\x00"
+#define GCR_DATA_MARK	"\xd5\xaa\x0b"
+#define GCR_SLIP_BYTE	"\x27\xaa"
+#define GCR_SELF_SYNC	"\x3f\xbf\x1e\x34\x3c\x3f"
+
+#define DATA_99		"\x99\x99"
+#define MFM_ADDR_MARK	"\x99\xa1\x99\xa1\x99\xa1\x99\xfe"
+#define MFM_INDEX_MARK	"\x99\xc2\x99\xc2\x99\xc2\x99\xfc"
+#define MFM_GAP_LEN	12
 
 struct floppy_state {
 	enum swim_state	state;
@@ -137,19 +193,20 @@ struct floppy_state {
 	struct timer_list timeout;
 	int	timeout_pending;
 	int	ejected;
-	struct wait_queue *wait;
+	wait_queue_head_t wait;
 	int	wanted;
-	int	in_media_bay;
+	struct device_node*	media_bay; /* NULL when not in bay */
 	char	dbdma_cmd_space[5 * sizeof(struct dbdma_cmd)];
 };
 
-static struct floppy_state floppy_states[1];
+static struct floppy_state floppy_states[MAX_FLOPPIES];
+static int floppy_count = 0;
 
 static unsigned short write_preamble[] = {
 	0x4e4e, 0x4e4e, 0x4e4e, 0x4e4e, 0x4e4e,	/* gap field */
 	0, 0, 0, 0, 0, 0,			/* sync field */
 	0x99a1, 0x99a1, 0x99a1, 0x99fb,		/* data address mark */
-	0x990f					/* init CRC generator */
+	0x990f					/* no escape for 512 bytes */
 };
 
 static unsigned short write_postamble[] = {
@@ -175,7 +232,7 @@ static void scan_timeout(unsigned long data);
 static void seek_timeout(unsigned long data);
 static void xfer_timeout(unsigned long data);
 static void swim3_interrupt(int irq, void *dev_id, struct pt_regs *regs);
-static void fd_dma_interrupt(int irq, void *dev_id, struct pt_regs *regs);
+/*static void fd_dma_interrupt(int irq, void *dev_id, struct pt_regs *regs);*/
 static int grab_drive(struct floppy_state *fs, enum swim_state state,
 		      int interruptible);
 static void release_drive(struct floppy_state *fs);
@@ -190,6 +247,7 @@ static ssize_t floppy_write(struct file *filp, const char *buf,
 			    size_t count, loff_t *ppos);
 static int floppy_check_change(kdev_t dev);
 static int floppy_revalidate(kdev_t dev);
+static int swim3_add_device(struct device_node *swims);
 int swim3_init(void);
 
 #define IOCTL_MODE_BIT	8
@@ -213,9 +271,9 @@ static void swim3_action(struct floppy_state *fs, int action)
 
 	swim3_select(fs, action);
 	udelay(1);
-	sw->select |= LSTRB; eieio();
+	out_8(&sw->select, sw->select | LSTRB);
 	udelay(2);
-	sw->select &= ~LSTRB; eieio();
+	out_8(&sw->select, sw->select & ~LSTRB);
 	udelay(1);
 	out_8(&sw->select, RELAX);
 }
@@ -226,7 +284,7 @@ static int swim3_readbit(struct floppy_state *fs, int bit)
 	int stat;
 
 	swim3_select(fs, bit);
-	udelay(1);
+	udelay(10);
 	stat = in_8(&sw->status);
 	out_8(&sw->select, RELAX);
 	return (stat & DATA) == 0;
@@ -234,13 +292,19 @@ static int swim3_readbit(struct floppy_state *fs, int bit)
 
 static void do_fd_request(void)
 {
-	start_request(&floppy_states[0]);
+	int i;
+	for(i=0;i<floppy_count;i++)
+	{
+		if (floppy_states[i].media_bay &&
+			check_media_bay(floppy_states[i].media_bay, MB_FD))
+			continue;
+		start_request(&floppy_states[i]);
+	}
 	sti();
 }
 
 static void start_request(struct floppy_state *fs)
 {
-	int drive;
 	unsigned long x;
 
 	if (fs->state == idle && fs->wanted) {
@@ -261,11 +325,6 @@ static void start_request(struct floppy_state *fs)
 		       CURRENT->rq_status, CURRENT->errors, CURRENT->current_nr_sectors);
 #endif
 
-		drive = MINOR(CURRENT->rq_dev);
-		if (drive != 0) {
-			end_request(0);
-			continue;
-		}
 		if (CURRENT->sector < 0 || CURRENT->sector >= fs->total_secs) {
 			end_request(0);
 			continue;
@@ -322,9 +381,9 @@ static inline void scan_track(struct floppy_state *fs)
 
 	swim3_select(fs, READ_DATA_0);
 	xx = sw->intr;		/* clear SEEN_SECTOR bit */
-	out_8(&sw->control_bis, SCAN_TRACK);
+	out_8(&sw->control_bis, DO_ACTION);
 	/* enable intr when track found */
-	out_8(&sw->intr_enable, ERROR | SEEN_SECTOR);
+	out_8(&sw->intr_enable, ERROR_INTR | SEEN_SECTOR);
 	set_timeout(fs, HZ, scan_timeout);	/* enable timeout */
 }
 
@@ -343,7 +402,7 @@ static inline void seek_track(struct floppy_state *fs, int n)
 	swim3_select(fs, STEP);
 	out_8(&sw->control_bis, DO_SEEK);
 	/* enable intr when seek finished */
-	out_8(&sw->intr_enable, ERROR | SEEK_DONE);
+	out_8(&sw->intr_enable, ERROR_INTR | SEEK_DONE);
 	set_timeout(fs, HZ/2, seek_timeout);	/* enable timeout */
 }
 
@@ -378,7 +437,7 @@ static inline void setup_transfer(struct floppy_state *fs)
 	swim3_select(fs, fs->head? READ_DATA_1: READ_DATA_0);
 	out_8(&sw->sector, fs->req_sector);
 	out_8(&sw->nsect, n);
-	out_8(&sw->ssize, 0);
+	out_8(&sw->gap3, 0);
 	st_le32(&dr->cmdptr, virt_to_bus(cp));
 	if (CURRENT->cmd == WRITE) {
 		/* Set up 3 dma commands: write preamble, data, postamble */
@@ -394,9 +453,9 @@ static inline void setup_transfer(struct floppy_state *fs)
 	out_le16(&cp->command, DBDMA_STOP);
 	out_le32(&dr->control, (RUN << 16) | RUN);
 	out_8(&sw->control_bis,
-	      (CURRENT->cmd == WRITE? WRITE_SECTORS: 0) | SCAN_TRACK);
+	      (CURRENT->cmd == WRITE? WRITE_SECTORS: 0) | DO_ACTION);
 	/* enable intr when transfer complete */
-	out_8(&sw->intr_enable, ERROR | TRANSFER_DONE);
+	out_8(&sw->intr_enable, ERROR_INTR | TRANSFER_DONE);
 	set_timeout(fs, 2*HZ, xfer_timeout);	/* enable timeout */
 }
 
@@ -438,8 +497,8 @@ static void act(struct floppy_state *fs)
 		case settling:
 			/* wait for SEEK_COMPLETE to become true */
 			swim3_select(fs, SEEK_COMPLETE);
-			udelay(1);
-			out_8(&sw->intr_enable, ERROR | DATA_CHANGED);
+			udelay(10);
+			out_8(&sw->intr_enable, ERROR_INTR | DATA_CHANGED);
 			in_8(&sw->intr);	/* clear DATA_CHANGED */
 			if (in_8(&sw->status) & DATA) {
 				/* seek_complete is not yet true */
@@ -481,7 +540,7 @@ static void scan_timeout(unsigned long data)
 	volatile struct swim3 *sw = fs->swim3;
 
 	fs->timeout_pending = 0;
-	out_8(&sw->control_bic, SCAN_TRACK);
+	out_8(&sw->control_bic, DO_ACTION);
 	out_8(&sw->select, RELAX);
 	out_8(&sw->intr_enable, 0);
 	fs->cur_cyl = -1;
@@ -531,7 +590,7 @@ static void xfer_timeout(unsigned long data)
 	fs->timeout_pending = 0;
 	st_le32(&dr->control, RUN << 16);
 	out_8(&sw->intr_enable, 0);
-	out_8(&sw->control_bic, WRITE_SECTORS | SCAN_TRACK);
+	out_8(&sw->control_bic, WRITE_SECTORS | DO_ACTION);
 	out_8(&sw->select, RELAX);
 	if (CURRENT->cmd == WRITE)
 		++cp;
@@ -560,15 +619,15 @@ static void swim3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	err = in_8(&sw->error);
 	intr = in_8(&sw->intr);
 #if 0
-	printk(KERN_DEBUG "swim3 intr state=%d intr=%x err=%x\n", fs->state, intr, err);
+	printk("swim3 intr state=%d intr=%x err=%x\n", fs->state, intr, err);
 #endif
-	if ((intr & ERROR) && fs->state != do_transfer)
+	if ((intr & ERROR_INTR) && fs->state != do_transfer)
 		printk(KERN_ERR "swim3_interrupt, state=%d, cmd=%x, intr=%x, err=%x\n",
 		       fs->state, CURRENT->cmd, intr, err);
 	switch (fs->state) {
 	case locating:
 		if (intr & SEEN_SECTOR) {
-			out_8(&sw->control_bic, SCAN_TRACK);
+			out_8(&sw->control_bic, DO_ACTION);
 			out_8(&sw->select, RELAX);
 			out_8(&sw->intr_enable, 0);
 			del_timer(&fs->timeout);
@@ -616,13 +675,13 @@ static void swim3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		act(fs);
 		break;
 	case do_transfer:
-		if ((intr & (ERROR | TRANSFER_DONE)) == 0)
+		if ((intr & (ERROR_INTR | TRANSFER_DONE)) == 0)
 			break;
 		dr = fs->dma;
 		cp = fs->dma_cmd;
 		st_le32(&dr->control, RUN << 16);
 		out_8(&sw->intr_enable, 0);
-		out_8(&sw->control_bic, WRITE_SECTORS | SCAN_TRACK);
+		out_8(&sw->control_bic, WRITE_SECTORS | DO_ACTION);
 		out_8(&sw->select, RELAX);
 		del_timer(&fs->timeout);
 		fs->timeout_pending = 0;
@@ -630,7 +689,7 @@ static void swim3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 			++cp;
 		stat = ld_le16(&cp->xfer_status);
 		resid = ld_le16(&cp->res_count);
-		if (intr & ERROR) {
+		if (intr & ERROR_INTR) {
 			n = fs->scount - 1 - resid / 512;
 			if (n > 0) {
 				CURRENT->sector += n;
@@ -685,9 +744,11 @@ static void swim3_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	}
 }
 
+/*
 static void fd_dma_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
 }
+*/
 
 static int grab_drive(struct floppy_state *fs, enum swim_state state,
 		      int interruptible)
@@ -740,8 +801,7 @@ static int fd_eject(struct floppy_state *fs)
 			break;
 		}
 		current->state = TASK_INTERRUPTIBLE;
-		current->timeout = jiffies + 1;
-		schedule();
+		schedule_timeout(1);
 	}
 	fs->ejected = 1;
 	release_drive(fs);
@@ -756,12 +816,20 @@ static int floppy_ioctl(struct inode *inode, struct file *filp,
 {
 	struct floppy_state *fs;
 	int err;
+	int devnum = MINOR(inode->i_rdev);
 
+	if (devnum >= floppy_count)
+		return -ENODEV;
+		
 	if (((cmd & 0x40) && !(filp && (filp->f_mode & IOCTL_MODE_BIT))) ||
 	    ((cmd & 0x80) && !suser()))
 		return -EPERM;
 
-	fs = &floppy_states[0];
+	fs = &floppy_states[devnum];
+
+	if (fs->media_bay && check_media_bay(fs->media_bay, MB_FD))
+		return -ENXIO;
+
 	switch (cmd) {
 	case FDEJECT:
 		if (fs->ref_count != 1)
@@ -781,19 +849,23 @@ static int floppy_open(struct inode *inode, struct file *filp)
 	struct floppy_state *fs;
 	volatile struct swim3 *sw;
 	int n, err;
+	int devnum = MINOR(inode->i_rdev);
 
-	if (MINOR(inode->i_rdev) != 0)
+	if (devnum >= floppy_count)
 		return -ENODEV;
-	fs = &floppy_states[0];
+	if (filp == 0)
+		return -EIO;
+		
+	fs = &floppy_states[devnum];
 	sw = fs->swim3;
 	err = 0;
 	if (fs->ref_count == 0) {
-		if (fs->in_media_bay && !check_media_bay(MB_FD))
+		if (fs->media_bay && check_media_bay(fs->media_bay, MB_FD))
 			return -ENXIO;
 		out_8(&sw->mode, 0x95);
 		out_8(&sw->control_bic, 0xff);
-		out_8(&sw->reg5, 0x28);
-		udelay(1);
+		out_8(&sw->setup, S_IBM_DRIVE | S_FCLK_DIV2);
+		udelay(10);
 		out_8(&sw->intr_enable, 0);
 		out_8(&sw->control_bis, DRIVE_ENABLE | INTR_ENABLE);
 		swim3_action(fs, MOTOR_ON);
@@ -807,8 +879,7 @@ static int floppy_open(struct inode *inode, struct file *filp)
 				break;
 			}
 			current->state = TASK_INTERRUPTIBLE;
-			current->timeout = jiffies + 1;
-			schedule();
+			schedule_timeout(1);
 		}
 		if (err == 0 && (swim3_readbit(fs, SEEK_COMPLETE) == 0
 				 || swim3_readbit(fs, DISK_IN) == 0))
@@ -818,14 +889,14 @@ static int floppy_open(struct inode *inode, struct file *filp)
 	} else if (fs->ref_count == -1 || filp->f_flags & O_EXCL)
 		return -EBUSY;
 
-	if (err == 0 && filp && (filp->f_flags & O_NDELAY) == 0
+	if (err == 0 && (filp->f_flags & O_NDELAY) == 0
 	    && (filp->f_mode & 3)) {
 		check_disk_change(inode->i_rdev);
 		if (fs->ejected)
 			err = -ENXIO;
 	}
 
-	if (err == 0 && filp && (filp->f_flags & (O_WRONLY | O_RDWR))) {
+	if (err == 0 && (filp->f_mode & 2)) {
 		if (fs->write_prot < 0)
 			fs->write_prot = swim3_readbit(fs, WRITE_PROT);
 		if (fs->write_prot)
@@ -858,9 +929,11 @@ static int floppy_release(struct inode *inode, struct file *filp)
 {
 	struct floppy_state *fs;
 	volatile struct swim3 *sw;
+	int devnum = MINOR(inode->i_rdev);
+
+	if (devnum >= floppy_count)
+		return -ENODEV;
 
-	if (MINOR(inode->i_rdev) != 0)
-		return -ENXIO;
 	/*
 	 * If filp is NULL, we're being called from blkdev_release
 	 * or after a failed mount attempt.  In the former case the
@@ -870,7 +943,7 @@ static int floppy_release(struct inode *inode, struct file *filp)
 	if (filp && (filp->f_mode & (2 | OPEN_WRITE_BIT)))
 		block_fsync (filp, filp->f_dentry);
 
-	fs = &floppy_states[0];
+	fs = &floppy_states[devnum];
 	sw = fs->swim3;
 	if (fs->ref_count > 0 && --fs->ref_count == 0) {
 		swim3_action(fs, MOTOR_OFF);
@@ -882,10 +955,12 @@ static int floppy_release(struct inode *inode, struct file *filp)
 static int floppy_check_change(kdev_t dev)
 {
 	struct floppy_state *fs;
+	int devnum = MINOR(dev);
 
-	if (MAJOR(dev) != MAJOR_NR || MINOR(dev) != 0)
+	if (MAJOR(dev) != MAJOR_NR || (devnum >= floppy_count))
 		return 0;
-	fs = &floppy_states[0];
+		
+	fs = &floppy_states[devnum];
 	return fs->ejected;
 }
 
@@ -894,10 +969,16 @@ static int floppy_revalidate(kdev_t dev)
 	struct floppy_state *fs;
 	volatile struct swim3 *sw;
 	int ret, n;
+	int devnum = MINOR(dev);
 
-	if (MAJOR(dev) != MAJOR_NR || MINOR(dev) != 0)
+	if (MAJOR(dev) != MAJOR_NR || (devnum >= floppy_count))
 		return 0;
-	fs = &floppy_states[0];
+
+	fs = &floppy_states[devnum];
+
+	if (fs->media_bay && check_media_bay(fs->media_bay, MB_FD))
+		return -ENXIO;
+
 	sw = fs->swim3;
 	grab_drive(fs, revalidating, 0);
 	out_8(&sw->intr_enable, 0);
@@ -911,8 +992,7 @@ static int floppy_revalidate(kdev_t dev)
 		if (signal_pending(current))
 			break;
 		current->state = TASK_INTERRUPTIBLE;
-		current->timeout = jiffies + 1;
-		schedule();
+		schedule_timeout(1);
 	}
 	ret = swim3_readbit(fs, SEEK_COMPLETE) == 0
 		|| swim3_readbit(fs, DISK_IN) == 0;
@@ -932,10 +1012,12 @@ static ssize_t floppy_read(struct file *filp, char *buf,
 {
 	struct inode *inode = filp->f_dentry->d_inode;
 	struct floppy_state *fs;
+	int devnum = MINOR(inode->i_rdev);
 
-	if (MINOR(inode->i_rdev) != 0)
+	if (devnum >= floppy_count)
 		return -ENODEV;
-	fs = &floppy_states[0];
+		
+	fs = &floppy_states[devnum];
 	if (fs->ejected)
 		return -ENXIO;
 	return block_read(filp, buf, count, ppos);
@@ -946,12 +1028,18 @@ static ssize_t floppy_write(struct file * filp, const char * buf,
 {
 	struct inode * inode = filp->f_dentry->d_inode;
 	struct floppy_state *fs;
+	int devnum = MINOR(inode->i_rdev);
 
-	if (MINOR(inode->i_rdev) != 0)
+	if (devnum >= floppy_count)
 		return -ENODEV;
-	fs = &floppy_states[0];
+	check_disk_change(inode->i_rdev);
+	fs = &floppy_states[devnum];
 	if (fs->ejected)
 		return -ENXIO;
+	if (fs->write_prot < 0)
+		fs->write_prot = swim3_readbit(fs, WRITE_PROT);
+	if (fs->write_prot)
+		return -EROFS;
 	return block_write(filp, buf, count, ppos);
 }
 
@@ -978,50 +1066,73 @@ static struct file_operations floppy_fops = {
 
 int swim3_init(void)
 {
-	struct device_node *swims;
-	struct floppy_state *fs = &floppy_states[0];
-	int is_3400 = 0;
+	struct device_node *swim;
 
-	if (find_devices("media-bay") != NULL) {
-		/* assume this is a PB3400 */
-		swims = find_devices("floppy");
-		is_3400 = 1;
-	} else {
-		swims = find_devices("swim3");
+	swim = find_devices("floppy");
+	while (swim && (floppy_count < MAX_FLOPPIES))
+	{
+		swim3_add_device(swim);
+		swim = swim->next;
 	}
 
-	if (swims == NULL)
-		return 0;
+	swim = find_devices("swim3");
+	while (swim && (floppy_count < MAX_FLOPPIES))
+	{
+		swim3_add_device(swim);
+		swim = swim->next;
+	}
 
-	if (swims->next != NULL)
-		printk(KERN_ERR "Warning: only using first SWIM3 floppy controller\n");
-	if (swims->n_addrs != 2 || swims->n_intrs != 2) {
-		printk(KERN_ERR "swim3: expecting 2 addrs and 2 intrs! (%d, %d)\n",
-		       swims->n_addrs, swims->n_intrs);
+	if (floppy_count > 0)
+	{
+		if (register_blkdev(MAJOR_NR, "fd", &floppy_fops)) {
+			printk(KERN_ERR "Unable to get major %d for floppy\n",
+			       MAJOR_NR);
+			return -EBUSY;
+		}
+		blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
+		blksize_size[MAJOR_NR] = floppy_blocksizes;
+		blk_size[MAJOR_NR] = floppy_sizes;
+	}
+
+	return 0;
+}
+
+static int swim3_add_device(struct device_node *swim)
+{
+	struct device_node *mediabay;
+	struct floppy_state *fs = &floppy_states[floppy_count];
+	
+	if (swim->n_addrs < 2)
+	{
+		printk(KERN_INFO "swim3: expecting 2 addrs (n_addrs:%d, n_intrs:%d)\n",
+		       swim->n_addrs, swim->n_intrs);
 		return -EINVAL;
 	}
 
-	if (register_blkdev(MAJOR_NR, "fd", &floppy_fops)) {
-		printk(KERN_ERR "Unable to get major %d for floppy\n",
-		       MAJOR_NR);
-		return -EBUSY;
+	if (swim->n_intrs < 2)
+	{
+		printk(KERN_INFO "swim3: expecting 2 intrs (n_addrs:%d, n_intrs:%d)\n",
+		       swim->n_addrs, swim->n_intrs);
+		return -EINVAL;
 	}
-	blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
-	blksize_size[MAJOR_NR] = floppy_blocksizes;
-	blk_size[MAJOR_NR] = floppy_sizes;
 
+	mediabay = (strcasecmp(swim->parent->type, "media-bay") == 0) ? swim->parent : NULL;
+	if (mediabay == NULL)
+		feature_set(swim, FEATURE_SWIM3_enable);
+	
 	memset(fs, 0, sizeof(*fs));
 	fs->state = idle;
-	fs->swim3 = (volatile struct swim3 *) swims->addrs[0].address;
-	fs->dma = (struct dbdma_regs *) swims->addrs[1].address;
-	fs->swim3_intr = swims->intrs[0].line;
-	fs->dma_intr = swims->intrs[1].line;
+	fs->swim3 = (volatile struct swim3 *) ioremap(swim->addrs[0].address, 0x200);
+	fs->dma = (struct dbdma_regs *) ioremap(swim->addrs[1].address, 0x200);
+	fs->swim3_intr = swim->intrs[0].line;
+	fs->dma_intr = swim->intrs[1].line;
 	fs->cur_cyl = -1;
 	fs->cur_sector = -1;
 	fs->secpercyl = 36;
 	fs->secpertrack = 18;
 	fs->total_secs = 2880;
-	fs->in_media_bay = is_3400;
+	fs->media_bay = mediabay;
+	init_waitqueue_head(&fs->wait);
 
 	fs->dma_cmd = (struct dbdma_cmd *) DBDMA_ALIGN(fs->dbdma_cmd_space);
 	memset(fs->dma_cmd, 0, 2 * sizeof(struct dbdma_cmd));
@@ -1029,19 +1140,26 @@ int swim3_init(void)
 
 	if (request_irq(fs->swim3_intr, swim3_interrupt, 0, "SWIM3", fs)) {
 		printk(KERN_ERR "Couldn't get irq %d for SWIM3\n", fs->swim3_intr);
+		feature_clear(swim, FEATURE_SWIM3_enable);
 		return -EBUSY;
 	}
+/*
 	if (request_irq(fs->dma_intr, fd_dma_interrupt, 0, "SWIM3-dma", fs)) {
 		printk(KERN_ERR "Couldn't get irq %d for SWIM3 DMA",
 		       fs->dma_intr);
+		feature_clear(swim, FEATURE_SWIM3_enable);
 		return -EBUSY;
 	}
+*/
 
 	init_timer(&fs->timeout);
 
 	do_floppy = NULL;
 
-	printk(KERN_INFO "fd0: SWIM3 floppy controller\n");
+	printk(KERN_INFO "fd%d: SWIM3 floppy controller %s\n", floppy_count,
+		mediabay ? "in media bay" : "");
 
+	floppy_count++;
+	
 	return 0;
 }
