@@ -24,8 +24,8 @@
 
 #include <asm/ptrace.h>
 
-#ifdef CONFIG_MAGIC_SYSRQ
-int sysrq_enabled = 1;
+#ifdef CONFIG_APM
+#include <linux/apm_bios.h>
 #endif
 
 extern void wakeup_bdflush(int);
@@ -33,9 +33,6 @@ extern void reset_vc(unsigned int);
 extern int console_loglevel;
 extern struct vfsmount *vfsmntlist;
 
-/* Machine specific power off function */
-void (*sysrq_power_off)(void) = NULL;
-
 /* Send a signal to all user processes */
 
 static void send_sig_all(int sig, int even_init)
@@ -43,7 +40,7 @@ static void send_sig_all(int sig, int even_init)
 	struct task_struct *p;
 
 	for_each_task(p) {
-		if (p->pid && p->mm != &init_mm) {	    /* Not swapper nor kernel thread */
+		if (p->mm) {				    /* Not swapper nor kernel thread */
 			if (p->pid == 1 && even_init)	    /* Ugly hack to kill init */
 				p->pid = 0x8000;
 			force_sig(sig, p);
@@ -85,12 +82,12 @@ void handle_sysrq(int key, struct pt_regs *pt_regs,
 		printk("Resetting\n");
 		machine_restart(NULL);
 		break;
+#ifdef CONFIG_APM
 	case 'o':					    /* O -- power off */
-		if (sysrq_power_off) {
-			printk("Power off\n");
-			sysrq_power_off();
-		}
+		printk("Power off\n");
+		apm_power_off();
 		break;
+#endif
 	case 's':					    /* S -- emergency sync */
 		printk("Emergency Sync\n");
 		emergency_sync_scheduled = EMERG_SYNC;
@@ -140,10 +137,11 @@ void handle_sysrq(int key, struct pt_regs *pt_regs,
 		if (tty)
 			printk("saK ");
 #endif
-		printk("Boot ");
-		if (sysrq_power_off)
-			printk("Off ");
-		printk("Sync Unmount showPc showTasks showMem loglevel0-8 tErm kIll killalL\n");
+		printk("Boot "
+#ifdef CONFIG_APM
+		       "Off "
+#endif
+		       "Sync Unmount showPc showTasks showMem loglevel0-8 tErm kIll killalL\n");
 		/* Don't use 'A' as it's handled specially on the Sparc */
 	}
 
@@ -152,15 +150,6 @@ void handle_sysrq(int key, struct pt_regs *pt_regs,
 
 /* Aux routines for the syncer */
 
-static void all_files_read_only(void)	    /* Kill write permissions of all files */
-{
-	struct file *file;
-
-	for (file = inuse_filps; file; file = file->f_next)
-		if (file->f_dentry && file->f_count && S_ISREG(file->f_dentry->d_inode->i_mode))
-			file->f_mode &= ~2;
-}
-
 static int is_local_disk(kdev_t dev)	    /* Guess if the device is a local hard drive */
 {
 	unsigned int major = MAJOR(dev);
@@ -194,6 +183,7 @@ static void go_sync(kdev_t dev, int remount_flag)
 		struct super_block *sb = get_super(dev);
 		struct vfsmount *vfsmnt;
 		int ret, flags;
+		struct list_head *p;
 
 		if (!sb) {
 			printk("Superblock not found\n");
@@ -203,6 +193,15 @@ static void go_sync(kdev_t dev, int remount_flag)
 			printk("R/O\n");
 			return;
 		}
+
+		file_list_lock();
+		for (p = sb->s_files.next; p != &sb->s_files; p = p->next) {
+			struct file *file = list_entry(p, struct file, f_list);
+			if (file->f_dentry && file_count(file)
+				&& S_ISREG(file->f_dentry->d_inode->i_mode))
+				file->f_mode &= ~2;
+		}
+		file_list_unlock();
 		DQUOT_OFF(dev);
 		fsync_dev(dev);
 		flags = MS_RDONLY;
@@ -242,9 +241,6 @@ void do_emergency_sync(void)
 	remount_flag = (emergency_sync_scheduled == EMERG_REMOUNT);
 	emergency_sync_scheduled = 0;
 
-	if (remount_flag)
-		all_files_read_only();
-
 	for (mnt = vfsmntlist; mnt; mnt = mnt->mnt_next)
 		if (is_local_disk(mnt->mnt_dev))
 			go_sync(mnt->mnt_dev, remount_flag);
