@@ -90,6 +90,7 @@
 #include <linux/errno.h>
 #include <linux/ioport.h>
 #include <linux/netdevice.h>
+#include <linux/spinlock.h>
 
 #undef COSA_SLOW_IO	/* for testing purposes only */
 #undef REALLY_SLOW_IO
@@ -97,7 +98,6 @@
 #include <asm/io.h>
 #include <asm/dma.h>
 #include <asm/byteorder.h>
-#include <asm/spinlock.h>
 
 #include "syncppp.h"
 #include "cosa.h"
