@@ -60,8 +60,6 @@
  *	12/19/98 bv, v1.02a Use spinlocks for 2.1.95 and up.
  **************************************************************************/
 
-#include <linux/config.h>
-
 #define ULONG   unsigned long
 #define PVOID   void *
 #define USHORT  unsigned short
@@ -78,14 +76,6 @@
 #define U32     unsigned long
 #endif
 
-#ifndef VIRT_TO_BUS
-#define VIRT_TO_BUS(x)           (unsigned long)virt_to_bus((void *) x)
-#endif
-
-#ifndef VIRT_TO_BUS
-#define VIRT_TO_BUS(x)           (unsigned int)virt_to_bus((void *) x)
-#endif
-
 #ifndef NULL
 #define NULL     0		/* zero          */
 #endif
