@@ -33,7 +33,7 @@
 #define SNDCARD_WAVEFRONT               41
 #define SNDCARD_OPL3SA2                 42
 #define SNDCARD_OPL3SA2_MPU             43
-#define SNDCARD_WAVEARTIST		44
+#define SNDCARD_WAVEARTIST              44	/* Rebel Waveartist */
 #define SNDCARD_OPL3SA2_MSS             45	/* Originally missed */
 #define SNDCARD_AD1816                  88
 
@@ -222,9 +222,9 @@ struct audio_operations
 	int parent_dev;		/* 0 -> no parent, 1 to n -> parent=parent_dev+1 */
 
 	/* fields formerly in dmabuf.c */
-	struct wait_queue *in_sleeper;
-	struct wait_queue *out_sleeper;
-	struct wait_queue *poll_sleeper;
+	wait_queue_head_t in_sleeper;
+	wait_queue_head_t out_sleeper;
+	wait_queue_head_t poll_sleeper;
 
 	/* fields formerly in audio.c */
 	int audio_mode;
