@@ -138,12 +138,11 @@
  *	This happens in pcm_copy_{in,out}().
  */
 
-#include <linux/config.h>
 #include <linux/module.h>
 #include <linux/stddef.h>
+#include <linux/spinlock.h>
 #include <asm/fixmap.h>
 #include <asm/sgi-cobalt.h>
-#include <asm/spinlock.h>
 
 #include "sound_config.h"
 
@@ -1453,7 +1452,7 @@ typedef enum vwsnd_port_flags {
 typedef struct vwsnd_port {
 
 	spinlock_t	lock;
-	struct wait_queue *queue;
+	wait_queue_head_t queue;
 	vwsnd_port_swstate_t swstate;
 	vwsnd_port_hwstate_t hwstate;
 	vwsnd_port_flags_t flags;
@@ -1507,7 +1506,7 @@ typedef struct vwsnd_dev {
 	struct semaphore io_sema;
 	struct semaphore mix_sema;
 	mode_t		open_mode;
-	struct wait_queue *open_wait;
+	wait_queue_head_t open_wait;
 
 	lithium_t	lith;
 
@@ -1826,7 +1825,7 @@ static void pcm_shutdown_port(vwsnd_dev_t *devc,
 {
 	unsigned long flags;
 	vwsnd_port_hwstate_t hwstate;
-	struct wait_queue wait = { current, NULL };
+	DECLARE_WAITQUEUE(wait, current);
 
 	aport->swstate = SW_INITIAL;
 	add_wait_queue(&aport->queue, &wait);
@@ -2194,7 +2193,7 @@ static void pcm_flush_frag(vwsnd_dev_t *devc)
 static void pcm_write_sync(vwsnd_dev_t *devc)
 {
 	vwsnd_port_t *wport = &devc->wport;
-	struct wait_queue wait = { current, NULL };
+	DECLARE_WAITQUEUE(wait, current);
 	unsigned long flags;
 	vwsnd_port_hwstate_t hwstate;
 
@@ -2286,7 +2285,7 @@ static ssize_t vwsnd_audio_do_read(struct file *file,
 		return -EFAULT;
 	ret = 0;
 	while (count) {
-		struct wait_queue wait = { current, NULL };
+		DECLARE_WAITQUEUE(wait, current);
 		add_wait_queue(&rport->queue, &wait);
 		current->state = TASK_INTERRUPTIBLE;
 		while ((nb = swb_inc_u(rport, 0)) == 0) {
@@ -2362,7 +2361,7 @@ static ssize_t vwsnd_audio_do_write(struct file *file,
 		return -EFAULT;
 	ret = 0;
 	while (count) {
-		struct wait_queue wait = { current, NULL };
+		DECLARE_WAITQUEUE(wait, current);
 		add_wait_queue(&wport->queue, &wait);
 		current->state = TASK_INTERRUPTIBLE;
 		while ((nb = swb_inc_u(wport, 0)) == 0) {
