@@ -23,10 +23,7 @@
  * uhci.c. Should have the correct interface now. 
  * (6/6/99)
  *
- * version 0.5 driver now generates a tty instead of a simple character
- * device
- *
- * version 0.3: Mayor changes. Changed Bulk transfer to interrupt based
+ * version 0.3: Major changes. Changed Bulk transfer to interrupt based
  * transfer. Using FIFO Buffers now. Consistent handling of open/close
  * file state and detected/nondetected device. File operations behave
  * according to this. Driver is able to send+receive now! Heureka!
@@ -59,8 +56,8 @@
 #include <linux/tty_flip.h>
 #include <linux/tty.h>
 #include <linux/module.h>
-//#include <sys/ioctl.h>
-#include <asm/spinlock.h>
+#include <linux/spinlock.h>
+
 #include "usb.h"
 
 #define NR_PORTS 3
@@ -256,7 +253,7 @@ static int acm_write_irq(int state, void *__buffer, int count, void *dev_id)
 static int rs_open(struct tty_struct *tty, struct file * filp) 
 {
 	struct acm_state *acm;
-
+	int ret;
 	
 	info("USB_FILE_OPEN\n");
 
@@ -275,10 +272,12 @@ static int rs_open(struct tty_struct *tty, struct file * filp)
 	acm->active=1;
  
 	/*Start reading from the device*/
-	acm->ctrltransfer=usb_request_irq(acm->dev,acm->ctrlpipe, acm_irq, acm->ctrlinterval, acm);
-	
+	ret = usb_request_irq(acm->dev,acm->ctrlpipe, acm_irq, acm->ctrlinterval, acm, &acm->ctrltransfer);
+	if (ret) {
+		printk (KERN_WARNING "usb-acm: usb_request_irq failed (0x%x)\n", ret);
+	}
 	acm->reading=1;
-	acm->readtransfer=usb_request_bulk(acm->dev,acm->readpipe, acm_read_irq, acm->readbuffer, acm->readsize, acm );
+	acm->readtransfer=usb_request_bulk(acm->dev,acm->readpipe, acm_read_irq, acm->readbuffer, acm->readsize, acm);
 	
 	Set_Control_Line_Status (CTRL_STAT_DTR | CTRL_STAT_RTS, acm);
 				                  
@@ -310,7 +309,7 @@ static void rs_close(struct tty_struct *tty, struct file * filp)
 		usb_terminate_bulk(acm->dev, acm->readtransfer);
 		acm->reading=0;
 	}
-//  usb_release_irq(acm->dev,acm->ctrltransfer);
+//  usb_release_irq(acm->dev,acm->ctrltransfer, acm->ctrlpipe);
 	
 	acm->active=0;
 }
@@ -485,6 +484,7 @@ static int acm_probe(struct usb_device *dev)
 	struct usb_interface_descriptor *interface;
 	struct usb_endpoint_descriptor *endpoint;
 	int cfgnum,acmno;
+	int swapped=0;
 	
 	info("acm_probe\n");
 	
@@ -500,14 +500,13 @@ static int acm_probe(struct usb_device *dev)
             dev->descriptor.bDeviceProtocol != 0)
 		return -1;
 
-	/* Now scan all configs for a ACM configuration */
+#define IFCLASS(if) ((if->bInterfaceClass << 24) | (if->bInterfaceSubClass << 16) | (if->bInterfaceProtocol << 8) | (if->bNumEndpoints))
+
+	/* Now scan all configs for a ACM configuration*/
 	for (cfgnum=0;cfgnum<dev->descriptor.bNumConfigurations;cfgnum++) {
 		/* The first one should be Communications interface? */
 		interface = &dev->config[cfgnum].interface[0].altsetting[0];
-		if (interface->bInterfaceClass != 2 ||
-		    interface->bInterfaceSubClass != 2 ||
-		    interface->bInterfaceProtocol != 1 ||
-	    	    interface->bNumEndpoints != 1)
+		if (IFCLASS(interface) != 0x02020101)
 			continue;
 
 		/*Which uses an interrupt input */
@@ -519,19 +518,20 @@ static int acm_probe(struct usb_device *dev)
 		/* The second one should be a Data interface? */
 		interface = &dev->config[cfgnum].interface[1].altsetting[0];
 		if (interface->bInterfaceClass != 10 ||
-		    interface->bInterfaceSubClass != 0 ||
-		    interface->bInterfaceProtocol != 0 ||
-	    	    interface->bNumEndpoints != 2)
+		    interface->bNumEndpoints != 2)
 			continue;
 
+		if ((endpoint->bEndpointAddress & 0x80) == 0x80)
+			swapped = 1;
+
 		/*With a bulk input */
-		endpoint = &interface->endpoint[0];
+		endpoint = &interface->endpoint[0^swapped];
 		if ((endpoint->bEndpointAddress & 0x80) != 0x80 ||
 		    (endpoint->bmAttributes & 3) != 2)
 			continue;
 			
 		/*And a bulk output */
-		endpoint = &interface->endpoint[1];
+		endpoint = &interface->endpoint[1^swapped];
 		if ((endpoint->bEndpointAddress & 0x80) == 0x80 ||
 		    (endpoint->bmAttributes & 3) != 2)
 			continue;
@@ -542,30 +542,30 @@ static int acm_probe(struct usb_device *dev)
 		acm->dev=dev;
 		dev->private=acm;
 
-		acm->readendp=dev->config[cfgnum].interface[1].altsetting[0].endpoint[0].bEndpointAddress;
+		acm->readendp=dev->config[cfgnum].interface[1].altsetting[0].endpoint[0^swapped].bEndpointAddress;
 		acm->readpipe=usb_rcvbulkpipe(dev,acm->readendp);
-		acm->readbuffer=kmalloc(acm->readsize=dev->config[cfgnum].interface[1].altsetting[0].endpoint[0].wMaxPacketSize,GFP_KERNEL);
+		acm->readbuffer=kmalloc(acm->readsize=dev->config[cfgnum].interface[1].altsetting[0].endpoint[0^swapped].wMaxPacketSize,GFP_KERNEL);
 		acm->reading=0;
 		if (!acm->readbuffer) {
 			printk("ACM: Couldn't allocate readbuffer\n");
 			return -1;
 		}
-		
-		acm->writeendp=dev->config[cfgnum].interface[1].altsetting[0].endpoint[1].bEndpointAddress;
+
+		acm->writeendp=dev->config[cfgnum].interface[1].altsetting[0].endpoint[1^swapped].bEndpointAddress;
 		acm->writepipe=usb_sndbulkpipe(dev,acm->writeendp);
-		acm->writebuffer=kmalloc(acm->writesize=dev->config[cfgnum].interface[1].altsetting[0].endpoint[1].wMaxPacketSize, GFP_KERNEL);
+		acm->writebuffer=kmalloc(acm->writesize=dev->config[cfgnum].interface[1].altsetting[0].endpoint[1^swapped].wMaxPacketSize, GFP_KERNEL);
 		acm->writing=0;
 		if (!acm->writebuffer) {
 			printk("ACM: Couldn't allocate writebuffer\n");
 			kfree(acm->readbuffer);
 			return -1;
 		}
-		
+
 		acm->ctrlendp=dev->config[cfgnum].interface[0].altsetting[0].endpoint[0].bEndpointAddress;
 		acm->ctrlpipe=usb_rcvctrlpipe(acm->dev,acm->ctrlendp);
 		acm->ctrlinterval=dev->config[cfgnum].interface[0].altsetting[0].endpoint[0].bInterval;
 
-		acm->present=1;
+		acm->present=1;				
 		MOD_INC_USE_COUNT;
 		return 0;
 	}
@@ -593,7 +593,7 @@ static void acm_disconnect(struct usb_device *dev)
 		usb_terminate_bulk(acm->dev, acm->readtransfer);
 		acm->reading=0;
 	}
-//  usb_release_irq(acm->dev,acm->ctrltransfer);
+//  usb_release_irq(acm->dev,acm->ctrltransfer, acm->ctrlpipe);
 	//BUG: What to do if a device is open?? Notify process or not allow cleanup?
 	acm->active=0;
 	acm->present=0;
@@ -660,7 +660,10 @@ int usb_acm_init(void)
 	acm_tty_driver.read_proc = NULL; //rs_read_proc;
 	acm_tty_driver.chars_in_buffer = rs_chars_in_buffer;
 	acm_tty_driver.flush_buffer = NULL; //rs_flush_buffer;
-	tty_register_driver(&acm_tty_driver);
+	if (tty_register_driver(&acm_tty_driver)) {
+		printk( "acm: failed to register tty driver\n" );
+		return -EPERM;
+	}
 	
 	//REGISTER USB DRIVER
 	usb_register(&acm_driver);
