@@ -42,13 +42,13 @@ struct inode_operations coda_symlink_inode_operations = {
 	NULL,			/* rename */
 	coda_readlink,		/* readlink */
 	coda_follow_link,     	/* follow_link */
+	NULL,			/* get_block */
 	NULL,			/* readpage */
 	NULL,			/* writepage */
-	NULL,			/* bmap */
+	NULL,			/* flushpage */
 	NULL,			/* truncate */
 	NULL,            	/* permission */
 	NULL,                   /* smap */
-	NULL,                   /* update page */
         NULL                    /* revalidate */
 };
 
