@@ -11,6 +11,14 @@
 #include <asm/math_emu.h>
 #include <asm/segment.h>
 #include <asm/page.h>
+#include <asm/types.h>
+#include <linux/threads.h>
+
+/*
+ * Default implementation of macro that returns current
+ * instruction pointer ("program counter").
+ */
+#define current_text_addr() ({ void *pc; __asm__("movl $1f,%0\n1:":"=g" (pc)); pc; })
 
 /*
  *  CPU type and hardware bug flags. Kept separately for each CPU.
@@ -36,7 +44,7 @@ struct cpuinfo_x86 {
 	int	fdiv_bug;
 	int	f00f_bug;
 	int	coma_bug;
-	unsigned long loops_per_jiffy;
+	unsigned long loops_per_sec;
 	unsigned long *pgd_quick;
 	unsigned long *pte_quick;
 	unsigned long pgtable_cache_sz;
@@ -48,9 +56,6 @@ struct cpuinfo_x86 {
 #define X86_VENDOR_UMC 3
 #define X86_VENDOR_NEXGEN 4
 #define X86_VENDOR_CENTAUR 5
-#define X86_VENDOR_RISE 6
-#define X86_VENDOR_TRANSMETA 7
-#define X86_VENDOR_NSC 8
 #define X86_VENDOR_UNKNOWN 0xff
 
 /*
@@ -73,9 +78,9 @@ struct cpuinfo_x86 {
 #define X86_FEATURE_PGE		0x00002000	/* Page Global Enable */
 #define X86_FEATURE_MCA		0x00004000	/* Machine Check Architecture */
 #define X86_FEATURE_CMOV	0x00008000	/* CMOV instruction (FCMOVCC and FCOMI too if FPU present) */
-#define X86_FEATURE_PAT		0x00010000	/* Page Attribute Table */
+#define X86_FEATURE_PAT	0x00010000	/* Page Attribute Table */
 #define X86_FEATURE_PSE36	0x00020000	/* 36-bit PSEs */
-#define X86_FEATURE_PN		0x00040000	/* Processor serial number */
+#define X86_FEATURE_18		0x00040000
 #define X86_FEATURE_19		0x00080000
 #define X86_FEATURE_20		0x00100000
 #define X86_FEATURE_21		0x00200000
@@ -91,6 +96,7 @@ struct cpuinfo_x86 {
 #define X86_FEATURE_AMD3D	0x80000000
 
 extern struct cpuinfo_x86 boot_cpu_data;
+extern struct tss_struct init_tss[NR_CPUS];
 
 #ifdef __SMP__
 extern struct cpuinfo_x86 cpu_data[];
@@ -106,27 +112,6 @@ extern void identify_cpu(struct cpuinfo_x86 *);
 extern void print_cpu_info(struct cpuinfo_x86 *);
 extern void dodgy_tsc(void);
 
-/*
- * EFLAGS bits
- */
-#define X86_EFLAGS_CF   0x00000001 /* Carry Flag */
-#define X86_EFLAGS_PF   0x00000004 /* Parity Flag */
-#define X86_EFLAGS_AF   0x00000010 /* Auxillary carry Flag */
-#define X86_EFLAGS_ZF   0x00000040 /* Zero Flag */
-#define X86_EFLAGS_SF   0x00000080 /* Sign Flag */
-#define X86_EFLAGS_TF   0x00000100 /* Trap Flag */
-#define X86_EFLAGS_IF   0x00000200 /* Interrupt Flag */
-#define X86_EFLAGS_DF   0x00000400 /* Direction Flag */
-#define X86_EFLAGS_OF   0x00000800 /* Overflow Flag */
-#define X86_EFLAGS_IOPL 0x00003000 /* IOPL mask */
-#define X86_EFLAGS_NT   0x00004000 /* Nested Task */
-#define X86_EFLAGS_RF   0x00010000 /* Resume Flag */
-#define X86_EFLAGS_VM   0x00020000 /* Virtual Mode */
-#define X86_EFLAGS_AC   0x00040000 /* Alignment Check */
-#define X86_EFLAGS_VIF  0x00080000 /* Virtual Interrupt Flag */
-#define X86_EFLAGS_VIP  0x00100000 /* Virtual Interrupt Pending */
-#define X86_EFLAGS_ID   0x00200000 /* CPUID detection flag */
-
 /*
  *	Generic CPUID function
  */
@@ -137,93 +122,23 @@ extern inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
 		  "=b" (*ebx),
 		  "=c" (*ecx),
 		  "=d" (*edx)
-		: "0" (op));
+		: "a" (op)
+		: "cc");
 }
 
+
 /*
- *  * CPUID functions returning a single datum
- *   */
-static inline unsigned int cpuid_eax(unsigned int op)
-{
-	unsigned int eax;
-
-	__asm__("cpuid"
-	: "=a" (eax)
-	: "0" (op)
-	: "bx", "cx", "dx");
-	return eax;
-}   
-static inline unsigned int cpuid_ebx(unsigned int op)
-{       
-	unsigned int eax, ebx;
-
-	__asm__("cpuid"
-	: "=a" (eax), "=b" (ebx)
-	: "0" (op)
-	: "cx", "dx" );
-	return ebx;
-}   
-static inline unsigned int cpuid_ecx(unsigned int op)
-{ 
-	unsigned int eax, ecx;
-
-	__asm__("cpuid"
-	: "=a" (eax), "=c" (ecx)
-	: "0" (op)
-	: "bx", "dx" );
-	return ecx;
-}
-static inline unsigned int cpuid_edx(unsigned int op)
-{
-	unsigned int eax, edx;
-
-	__asm__("cpuid"
-	: "=a" (eax), "=d" (edx)
-	: "0" (op)
-	: "bx", "cx");
-	return edx;
-}
-/*
- *      Cyrix CPU configuration register indexes
+ * Intel CPU features in CR4
  */
-#define CX86_CCR0 0xc0
-#define CX86_CCR1 0xc1
-#define CX86_CCR2 0xc2
-#define CX86_CCR3 0xc3
-#define CX86_CCR4 0xe8
-#define CX86_CCR5 0xe9
-#define CX86_CCR6 0xea
-#define CX86_CCR7 0xeb
-#define CX86_DIR0 0xfe
-#define CX86_DIR1 0xff
-#define CX86_ARR_BASE 0xc4
-#define CX86_RCR_BASE 0xdc
-
-/*
- *      Cyrix CPU indexed register access macros
- */
-
-#define getCx86(reg) ({ outb((reg), 0x22); inb(0x23); })
-
-#define setCx86(reg, data) do { \
-	outb((reg), 0x22); \
-	outb((data), 0x23); \
-} while (0)
-
-/*
- *  * Intel CPU features in CR4
- *   */
-#define X86_CR4_VME     0x0001  /* enable vm86 extensions */
-#define X86_CR4_PVI     0x0002  /* virtual interrupts flag enable */
-#define X86_CR4_TSD     0x0004  /* disable time stamp at ipl 3 */
-#define X86_CR4_DE      0x0008  /* enable debugging extensions */
-#define X86_CR4_PSE     0x0010  /* enable page size extensions */
-#define X86_CR4_PAE     0x0020  /* enable physical address extensions */
-#define X86_CR4_MCE     0x0040  /* Machine check enable */
-#define X86_CR4_PGE     0x0080  /* enable global pages */
-#define X86_CR4_PCE     0x0100  /* enable performance counters at ipl 3 */
-#define X86_CR4_OSFXSR      0x0200  /* enable fast FPU save and restore */
-#define X86_CR4_OSXMMEXCPT  0x0400  /* enable unmasked SSE exceptions */
+#define X86_CR4_VME	0x0001	/* enable vm86 extensions */
+#define X86_CR4_PVI	0x0002	/* virtual interrupts flag enable */
+#define X86_CR4_TSD	0x0004	/* disable time stamp at ipl 3 */
+#define X86_CR4_DE	0x0008	/* enable debugging extensions */
+#define X86_CR4_PSE	0x0010	/* enable page size extensions */
+#define X86_CR4_PAE	0x0020	/* enable physical address extensions */
+#define X86_CR4_MCE	0x0040	/* Machine check enable */
+#define X86_CR4_PGE	0x0080	/* enable global pages */
+#define X86_CR4_PCE	0x0100	/* enable performance counters at ipl 3 */
 
 /*
  * Save the cr4 feature set we're using (ie
@@ -240,9 +155,45 @@ static inline void set_in_cr4 (unsigned long mask)
 		"orl %0,%%eax\n\t"
 		"movl %%eax,%%cr4\n"
 		: : "irg" (mask)
-		:"ax"); 
+		:"ax");
 }
 
+static inline void clear_in_cr4 (unsigned long mask)
+{
+	mmu_cr4_features &= ~mask;
+	__asm__("movl %%cr4,%%eax\n\t"
+		"andl %0,%%eax\n\t"
+		"movl %%eax,%%cr4\n"
+		: : "irg" (~mask)
+		:"ax");
+}
+
+/*
+ *      Cyrix CPU configuration register indexes
+ */
+#define CX86_CCR0 0xc0
+#define CX86_CCR1 0xc1
+#define CX86_CCR2 0xc2
+#define CX86_CCR3 0xc3
+#define CX86_CCR4 0xe8
+#define CX86_CCR5 0xe9
+#define CX86_CCR6 0xea
+#define CX86_DIR0 0xfe
+#define CX86_DIR1 0xff
+#define CX86_ARR_BASE 0xc4
+#define CX86_RCR_BASE 0xdc
+
+/*
+ *      Cyrix CPU indexed register access macros
+ */
+
+#define getCx86(reg) ({ outb((reg), 0x22); inb(0x23); })
+
+#define setCx86(reg, data) do { \
+	outb((reg), 0x22); \
+	outb((data), 0x23); \
+} while (0)
+
 /*
  * Bus types (default is ISA, but people can check others with these..)
  */
@@ -270,6 +221,8 @@ extern unsigned int mca_pentium_flag;
  * Size of io_bitmap in longwords: 32 is ports 0-0x3ff.
  */
 #define IO_BITMAP_SIZE	32
+#define IO_BITMAP_OFFSET offsetof(struct tss_struct,io_bitmap)
+#define INVALID_IO_BITMAP_OFFSET 0x8000
 
 struct i387_hard_struct {
 	long	cwd;
@@ -306,7 +259,7 @@ typedef struct {
 	unsigned long seg;
 } mm_segment_t;
 
-struct thread_struct {
+struct tss_struct {
 	unsigned short	back_link,__blh;
 	unsigned long	esp0;
 	unsigned short	ss0,__ss0h;
@@ -314,7 +267,7 @@ struct thread_struct {
 	unsigned short	ss1,__ss1h;
 	unsigned long	esp2;
 	unsigned short	ss2,__ss2h;
-	unsigned long	cr3;
+	unsigned long	__cr3;
 	unsigned long	eip;
 	unsigned long	eflags;
 	unsigned long	eax,ecx,edx,ebx;
@@ -331,19 +284,43 @@ struct thread_struct {
 	unsigned short	ldt, __ldth;
 	unsigned short	trace, bitmap;
 	unsigned long	io_bitmap[IO_BITMAP_SIZE+1];
-	unsigned long	tr;
-	unsigned long	cr2, trap_no, error_code;
-	mm_segment_t	segment;
-/* debug registers */
-	long debugreg[8];  /* Hardware debugging registers */
-/* floating point info */
-	union i387_union i387;
-/* virtual 86 mode info */
-	struct vm86_struct * vm86_info;
-	unsigned long screen_bitmap;
-	unsigned long v86flags, v86mask, v86mode, saved_esp0;
+	/*
+	 * pads the TSS to be cacheline-aligned (size is 0x100)
+	 */
+	unsigned long __cacheline_filler[5];
 };
 
+struct thread_struct {
+	unsigned long	esp0;
+	unsigned long	eip;
+	unsigned long	esp;
+	unsigned long	fs;
+	unsigned long	gs;
+/* Hardware debugging registers */
+	unsigned long	debugreg[8];  /* %%db0-7 debug registers */
+/* fault info */
+	unsigned long	cr2, trap_no, error_code;
+/* floating point info */
+	union i387_union	i387;
+/* virtual 86 mode info */
+	struct vm86_struct	* vm86_info;
+	unsigned long		screen_bitmap;
+	unsigned long		v86flags, v86mask, v86mode, saved_esp0;
+/* IO permissions */
+	int		ioperm;
+	unsigned long	io_bitmap[IO_BITMAP_SIZE+1];
+};
+
+#define INIT_THREAD  {						\
+	0,							\
+	0, 0, 0, 0, 						\
+	{ [0 ... 7] = 0 },	/* debugging registers */	\
+	0, 0, 0,						\
+	{ { 0, }, },		/* 387 state */			\
+	0,0,0,0,0,0,						\
+	0,{~0,}			/* io permissions */		\
+}
+
 #define INIT_MMAP \
 { &init_mm, 0, 0, NULL, PAGE_SHARED, VM_READ | VM_WRITE | VM_EXEC, 1, NULL, NULL }
 
@@ -352,19 +329,15 @@ struct thread_struct {
 	sizeof(init_stack) + (long) &init_stack, /* esp0 */	\
 	__KERNEL_DS, 0, /* ss0 */				\
 	0,0,0,0,0,0, /* stack1, stack2 */			\
-	(long) &swapper_pg_dir - PAGE_OFFSET, /* cr3 */		\
+	0, /* cr3 */						\
 	0,0, /* eip,eflags */					\
 	0,0,0,0, /* eax,ecx,edx,ebx */				\
 	0,0,0,0, /* esp,ebp,esi,edi */				\
 	0,0,0,0,0,0, /* es,cs,ss */				\
 	0,0,0,0,0,0, /* ds,fs,gs */				\
-	_LDT(0),0, /* ldt */					\
-	0, 0x8000, /* tace, bitmap */				\
-	{~0, }, /* ioperm */					\
-	_TSS(0), 0, 0, 0, (mm_segment_t) { 0 }, /* obsolete */	\
-	{ 0, },							\
-	{ { 0, }, },  /* 387 state */				\
-	NULL, 0, 0, 0, 0, 0, /* vm86_info */			\
+	__LDT(0),0, /* ldt */					\
+	0, INVALID_IO_BITMAP_OFFSET, /* tace, bitmap */		\
+	{~0, } /* ioperm */					\
 }
 
 #define start_thread(regs, new_eip, new_esp) do {		\
@@ -379,14 +352,18 @@ struct thread_struct {
 } while (0)
 
 /* Forward declaration, a strange C thing */
+struct task_struct;
 struct mm_struct;
 
 /* Free all resources held by a thread. */
 extern void release_thread(struct task_struct *);
+/*
+ * create a kernel thread without removing it from tasklists
+ */
 extern int kernel_thread(int (*fn)(void *), void * arg, unsigned long flags);
 
 /* Copy and release all segment info associated with a VM */
-extern void copy_segments(int nr, struct task_struct *p, struct mm_struct * mm);
+extern void copy_segments(struct task_struct *p, struct mm_struct * mm);
 extern void release_segments(struct mm_struct * mm);
 extern void forget_segments(void);
 
@@ -394,7 +371,7 @@ extern void forget_segments(void);
  * FPU lazy state save handling..
  */
 #define save_fpu(tsk) do { \
-	asm volatile("fnsave %0\n\tfwait":"=m" (tsk->tss.i387)); \
+	asm volatile("fnsave %0\n\tfwait":"=m" (tsk->thread.i387)); \
 	tsk->flags &= ~PF_USEDFPU; \
 	stts(); \
 } while (0)
@@ -419,33 +396,11 @@ extern inline unsigned long thread_saved_pc(struct thread_struct *t)
 	return ((unsigned long *)t->esp)[3];
 }
 
+#define THREAD_SIZE (2*PAGE_SIZE)
 extern struct task_struct * alloc_task_struct(void);
 extern void free_task_struct(struct task_struct *);
 
 #define init_task	(init_task_union.task)
 #define init_stack	(init_task_union.stack)
 
-/* '6' because it used to be for P6 only, now supports P15 too */
-#define MICROCODE_IOCFREE _IO('6',0)
-
-/* physical layour of IA32 microcode chunks */
-struct microcode {
-	unsigned int hdrver;
-	unsigned int rev;
-	unsigned int date;
-	unsigned int sig;
-	unsigned int cksum;
-	unsigned int ldrver;
-	unsigned int pf;
-	unsigned int reserved[5];
-	unsigned int bits[500];
-};
-
-
-/* REP NOP (PAUSE) is a good thing to insert into busy-wait loops. */
-extern inline void rep_nop(void)
-{
-	__asm__ __volatile__("rep;nop");
-}
-
 #endif /* __ASM_I386_PROCESSOR_H */
