@@ -1,7 +1,7 @@
 #ifndef _IDE_H
 #define _IDE_H
 /*
- *  linux/drivers/block/ide.h
+ *  linux/include/linux/ide.h
  *
  *  Copyright (C) 1994-1998  Linus Torvalds & authors
  */
@@ -265,6 +265,7 @@ typedef struct ide_drive_s {
 #if FAKE_FDISK_FOR_EZDRIVE
 	unsigned remap_0_to_1	: 1;	/* flag: partitioned with ezdrive */
 #endif /* FAKE_FDISK_FOR_EZDRIVE */
+	unsigned ata_flash	: 1;	/* 1=present, 0=default */
 	byte		media;		/* disk, cdrom, tape, floppy, ... */
 	select_t	select;		/* basic drive/head select reg value */
 	byte		ctl;		/* "normal" value for IDE_CONTROL_REG */
@@ -317,7 +318,8 @@ typedef struct ide_drive_s {
 typedef enum {	ide_dma_read,	ide_dma_write,		ide_dma_begin,
 		ide_dma_end,	ide_dma_check,		ide_dma_on,
 		ide_dma_off,	ide_dma_off_quietly,	ide_dma_test_irq,
-		ide_dma_bad_drive,			ide_dma_good_drive
+		ide_dma_bad_drive,			ide_dma_good_drive,
+		ide_dma_lostirq,			ide_dma_timeout
 	} ide_dma_action_t;
 
 typedef int (ide_dmaproc_t)(ide_dma_action_t, ide_drive_t *);
@@ -388,6 +390,7 @@ typedef struct hwif_s {
 	unsigned	sharing_irq: 1;	/* 1 = sharing irq with another hwif */
 	unsigned	reset      : 1;	/* reset after probe */
 	unsigned	autodma    : 1;	/* automatically try to enable DMA at boot */
+	unsigned	udma_four  : 1;	/* 1=ATA-66 capable, 0=default */
 	byte		channel;	/* for dual-port chips: 0=primary, 1=secondary */
 	struct pci_dev	*pci_dev;	/* for pci chipsets */
 	ide_pci_devid_t	pci_devid;	/* for pci chipsets: {VID,DID} */
@@ -696,6 +699,7 @@ void ide_end_drive_cmd (ide_drive_t *drive, byte stat, byte err);
 int ide_wait_cmd (ide_drive_t *drive, int cmd, int nsect, int feature, int sectors, byte *buf);
 
 void ide_delay_50ms (void);
+int ide_config_drive_speed (ide_drive_t *drive, byte speed);
 
 /*
  * ide_system_bus_speed() returns what we think is the system VESA/PCI
@@ -755,6 +759,12 @@ void do_ide6_request (void);
 #if MAX_HWIFS > 7
 void do_ide7_request (void);
 #endif
+#if MAX_HWIFS > 8
+void do_ide8_request (void);
+#endif
+#if MAX_HWIFS > 9
+void do_ide9_request (void);
+#endif
 void ide_init_subdrivers (void);
 
 #ifndef _IDE_C
